/**
 * SoccerDB Widget v1.0 (2006-12-03)
 * by Mats Bovin
 * Copyright (c) 2006
 * http://www.mbovin.com/soccerdb
 */

// The server we're using
var gServerName = 'http://www.mbovin.com';

// State variables
var gCountryId	= null;
var gLeagueId	= null;
var gStageId	= null;
var gGroupId	= null;
var gMultiGroup	= null;
var gMatchDate	= null;

// HttpRequest variables
var gCountriesRequest;
var gLeaguesRequest;
var gTableRequest;
var gMatchesRequest;

// GUI
var gMatchesScrollbar;
var gMatchesScrollArea;


/**
 * Starting point of the widget. Called from body.onload()
 */
function setup()
{
	setupGUI();
	loadSettings();
	loadCountries();
	// Schedule views to be refershed every hour.
	setTimeout('refreshViews();', 3600000);
}

/**
 * Sets up GUI-stuff
 */
function setupGUI()
{
	var infoButton = 
		new AppleInfoButton(document.getElementById('infoButton'), 
			document.getElementById('frontDiv'), 'white', 'white', showPrefs);
	var glassDoneButton = 
		new AppleGlassButton(document.getElementById('doneButton'), 'Done', hidePrefs);

    gMatchesScrollbar = new AppleVerticalScrollbar(
        document.getElementById("matchesScrollbar"));
 
    gMatchesScrollArea = new AppleScrollArea(
        document.getElementById("matchesScrollArea"), gMatchesScrollbar);
 
	gMatchesScrollArea.scrollsHorizontally		= false;
	gMatchesScrollArea.singlepressScrollPixels	= 25;
}

/**
 * Flips to the backside of the widget
 */
function showPrefs()
{
	var frontDiv	= document.getElementById('frontDiv');
	var backDiv		= document.getElementById('backDiv');
	
	if (window.widget)
	{
		widget.prepareForTransition('ToBack');
	}
	
	frontDiv.style.display = 'none';
	backDiv.style.display  = 'block';

	if (window.widget)
	{
		// If window is smaller than 156, we need to make it bigger
		var resize = (window.outerHeight < 156);
		if (resize)
		{
			window.resizeTo(186, 156);
		}
		setTimeout('widget.performTransition();', 0);
		if (resize)
		{		
			// This is a workaround to make the entire widget redraw itself 
			setTimeout('window.resizeBy(0, 1);', 700);	
		}
	}
}

/**
 * Flips to the front.
 */
function hidePrefs()
{
	var frontDiv	= document.getElementById('frontDiv');
	var backDiv		= document.getElementById('backDiv');
	
	if (window.widget) 
	{
		widget.prepareForTransition('ToFront');
	}
	
	backDiv.style.display  = 'none';
	frontDiv.style.display = 'block';
	
	// Resize the widget to match the current table
	resizeToTable();
	
	if (window.widget)
	{
		setTimeout('widget.performTransition();', 0);
		// This is a workaround to make the entire widget redraw itself 
		setTimeout('window.resizeBy(0, 1);', 700);
	}
}

/**
 * Reloads the data of the table and matches views. 
 * The function then reschedules itself to be run
 * again in an hour.
 */
function refreshViews()
{
	var isBackVisible = 
		(document.getElementById('backDiv').style.display == 'block');

	// Refresh views if we've got a group and the backside is
	// not currently visible
	if ((gGroupId != null) && !isBackVisible)
	{
		loadTable();	
		loadMatches(gMatchDate);
	}
	
	// Run again in an hour
	setTimeout('refreshViews();', 3600000);
}

/**
 * Called when a new country is selected in the drop-down.
 */
function countrySelected(countryId)
{
	gCountryId = countryId;
	saveSetting('countryId', countryId);
	loadLeagues(countryId);
}

/**
 * Called when a new league is selected in the drop-down, or
 * implicitly when a new country is selected.
 */
function leagueSelected(groupStr)
{
	if (groupStr != '')
	{
		// groupStr (the value of the listboxitem) is
		// a semicolon-separated string 
		groupArr	= groupStr.split(';');
		gGroupId	= groupArr[0];
		gStageId	= groupArr[1];
		gLeagueId	= groupArr[2];
		gMultiGroup	= groupArr[3];
	
		saveSetting('groupId', gGroupId);
		
		loadTable();	
		loadMatches(null);

		// Display top-menu (it's not visible when the widget starts)
		document.getElementById('menuDiv').style.display = 'block';
	}
}

/**
 * Shows the specified tab (table or matches).
 */
function showTab(tabName)
{
	var tableDiv		= document.getElementById('tableDiv');
	var matchesDiv		= document.getElementById('matchesDiv');
	var tableButton		= document.getElementById('tableButton');
	var matchesButton	= document.getElementById('matchesButton');
	
	if (tabName == 'table')
	{
		tableDiv.style.display		= 'block';
		matchesDiv.style.display	= 'none';
		tableButton.className		= 'ButtonDiv ButtonDivActive';
		matchesButton.className		= 'ButtonDiv';		
	}
	else if (tabName == 'matches')
	{
		matchesDiv.style.display	= 'block';
		tableDiv.style.display		= 'none';
		matchesButton.className		= 'ButtonDiv ButtonDivActive';
		tableButton.className		= 'ButtonDiv';		
		gMatchesScrollArea.refresh();
	}
}

/**
 * Called when Next or Previous matches is clicked.
 */
function gotoMatchDate(delta)
{
	if (gMatchDate)
	{
		var date = new Date(gMatchDate.substr(0,4), 
							gMatchDate.substr(5,2)-1, 
							gMatchDate.substr(8,2));
		if (delta > 0) // Next week
		{
			date.setDate(date.getDate() + 7);
		}
		else if (delta < 0) // Previous week
		{
			date.setDate(date.getDate() - 7);
		}

		loadMatches(formatDate(date));
	}
}

/** 
 * Opens the currently selected league in the web browser.
 */
function openLeagueInBrowser()
{
	var url = (gStageId != null) ? 
		gServerName + '/soccerdb/league/latest.php?stageId=' + gStageId + '&groupId=' + gGroupId :
		gServerName + '/soccerdb/';
	
	openUrlInBrowser(url);
}

/**
 * Resizes the widget to match the size of the current league table.
 */
function resizeToTable()
{
	showTab('table');

	var tableDiv			= document.getElementById('tableDiv');
	var frontDiv			= document.getElementById('frontDiv');
	var matchesScrollArea	= document.getElementById('matchesScrollArea');
	var matchesScrollbar	= document.getElementById('matchesScrollbar');
	
	var tableHeight					= tableDiv.clientHeight;
	var widgetHeight				= tableHeight + 45;
	
	frontDiv.style.height			= widgetHeight + 'px';
	matchesScrollArea.style.height	= (widgetHeight - 58) + 'px';
	matchesScrollbar.style.height	= (widgetHeight - 62) + 'px';

	if (window.widget)
	{
		window.resizeTo(186, widgetHeight);
	}
}

/**
 * Fires request to load countries.
 */ 
function loadCountries()
{
	var countrySelect = document.getElementById('countrySelect');
	if (countrySelect.options.length <= 1)
	{
		gCountryRequest = new XMLHttpRequest();
		gCountryRequest.onreadystatechange = processCountriesRequestChange;
		gCountryRequest.open('GET', gServerName + '/soccerdb/export/countrylist.php?withLeagues=1', true);
		gCountryRequest.send("");
	}
}

/**
 * Checks if the country request is done.
 */
function processCountriesRequestChange() 
{
	// Only if req shows "loaded" and "ok"
	if ((gCountryRequest.readyState == 4) &&
		(gCountryRequest.status == 200)) 
	{
		populateCountrySelect(gCountryRequest.responseXML)
	}
}

/**
 * Populates the country drop-down.
 */
function populateCountrySelect(xmlDoc)
{
	var countrySelect = document.getElementById('countrySelect');
	
	// Clear select and add default options
	countrySelect.options.length = 0;
	countrySelect.options[0] = new Option('-Select country-', '-2');
	countrySelect.options[1] = new Option('International', '-1', false, (gCountryId == -1));
    
	// Loop over countries in the xml and add them to the select
	var items = xmlDoc.getElementsByTagName('country');
	for (var i = 0; i < items.length; i++) 
	{
		var countryId	= getAttributeValue(items[i], 'id');
		var countryName	= getChildNodeValue(items[i], 'name');

		countrySelect.options[countrySelect.options.length] = 
			new Option(countryName, countryId, false, (gCountryId == countryId));
	}
	
	// If a country is selected we should load its leagues
	if (gCountryId)
	{
		countrySelected(gCountryId);
	}
}

/**
 * Fires request to load leagues.
 */
function loadLeagues(countryId)
{
	// Clear the league drop-down
	var leagueSelect = document.getElementById('leagueSelect');
	leagueSelect.options.length = 0;
	leagueSelect.options[leagueSelect.options.length] = new Option('Loading...', '');
	
	gLeaguesRequest = new XMLHttpRequest();
	gLeaguesRequest.onreadystatechange = processLeaguesRequestChange;
	gLeaguesRequest.open('GET', gServerName + '/soccerdb/export/grouplist.php?countryId=' + countryId, true);
	gLeaguesRequest.send("");
}

/**
 * Checks if the league request is done.
 */
function processLeaguesRequestChange() 
{
	// Only if req shows "loaded" and "ok"
    if ((gLeaguesRequest.readyState == 4) &&
		(gLeaguesRequest.status == 200)) 
	{
		populateLeagueSelect(gLeaguesRequest.responseXML)
	}
}

/**
 * Populates the league drop-down.
 */
function populateLeagueSelect(xmlDoc)
{
	var leagueSelect = document.getElementById('leagueSelect');
	leagueSelect.options.length = 0;

	// Add league from xml
    var items = xmlDoc.getElementsByTagName('group');
	for (var i = 0; i < items.length; i++) 
	{
		var groupId		= getAttributeValue(items[i], 'id');
		var stageId		= getAttributeValue(items[i], 'stageid');
		var leagueId	= getAttributeValue(items[i], 'leagueid');
		var multiGroup	= getAttributeValue(items[i], 'multigroup');
		var name		= getChildNodeValue(items[i], 'name');
		var valueStr	= groupId + ';' + stageId + ';' + leagueId + ';' + multiGroup;

		leagueSelect.options[leagueSelect.options.length] = 
			new Option(name, valueStr, false, (groupId == gGroupId));
	}

	// Signal that a new league is selected to load table and matches
	leagueSelected(leagueSelect.options[leagueSelect.selectedIndex].value)
}

/**
 * Fires request to load the table.
 */
function loadTable()
{
	var dataDiv = document.getElementById('tableDataDiv');
	dataDiv.innerText = 'Loading...';
	
	// If there is only one group in the league we use leagueId,
	// otherwise we need to use groupId.
	var qsParam;
	if (gMultiGroup != 1)
	{
		qsParam = 'leagueId=' + gLeagueId;
	}
	else
	{
		qsParam = 'groupId=' + gGroupId;
	}
	
	gTableRequest = new XMLHttpRequest();
	gTableRequest.onreadystatechange = processTableRequestChange;
	gTableRequest.open('GET', gServerName + '/soccerdb/export/table.php?' + qsParam + '&columns=pos,team,played,pts,gdiff&cssLines=TableLine&shortName=1', true);
	gTableRequest.send("");
}

/**
 * Checks if the table request is done.
 */
function processTableRequestChange() 
{
	// Only if req shows "loaded" and "ok"
	if ((gTableRequest.readyState == 4) && 
		(gTableRequest.status == 200))
	{
		showTable(gTableRequest.responseXML)
	}
}

/**
 * Shows the table from the loaded xml (html).
 */
function showTable(xmlDoc)
{
	var tableHtmlNode =
		(xmlDoc.documentElement.getElementsByTagName('htmldata'))[0];
	var tableHtml = getCData(tableHtmlNode);
	
	var dataDiv = document.getElementById('tableDataDiv');
	dataDiv.innerHTML = tableHtml;
	
	// Resize the widget if the front is visible (this will happen 
	// during widget startup or if the user flips to front before
	// the league drop-down is finsihed loading).
	if (document.getElementById('frontDiv').style.display != 'none')
	{
		resizeToTable();
	}
}

/**
 * Fires request to load matches.
 */
function loadMatches(matchDate)
{
	var dataDiv = document.getElementById('matchesDataDiv');
	dataDiv.innerText = 'Loading...';
	
	// If there is only one group in the league we use leagueId,
	// otherwise we need to use groupId.
	var qsParam;
	if (gMultiGroup != 1)
	{
		qsParam = 'leagueId=' + gLeagueId;
	}
	else
	{
		qsParam = 'groupId=' + gGroupId;
	}
	
	var dateParam = matchDate ? '&date=' + matchDate : '';
	gMatchesRequest = new XMLHttpRequest();
	gMatchesRequest.onreadystatechange = processMatchesRequestChange;
	gMatchesRequest.open('GET', gServerName + '/soccerdb/export/matches.php?' + qsParam + dateParam + '&shortName=1&cssDate=MatchesDate', true);
	gMatchesRequest.send("");
}

/**
 * Checks if the matches request is done.
 */
function processMatchesRequestChange() 
{
	// Only if req shows "loaded" and "ok"
	if ((gMatchesRequest.readyState == 4) && 
		(gMatchesRequest.status == 200))
	{
		showMatches(gMatchesRequest.responseXML)
	}
}

/**
 * Shows the matches from the loaded xml (html).
 */
function showMatches(xmlDoc)
{
	gMatchDate = getAttributeValue(xmlDoc.documentElement, 'date');

	var matchesHtmlNode = (xmlDoc.documentElement.getElementsByTagName('htmldata'))[0];
	var matchesHtml = getCData(matchesHtmlNode);
	var dataDiv = document.getElementById('matchesDataDiv');
	dataDiv.innerHTML = matchesHtml;
	gMatchesScrollArea.refresh();
}

/**
 * Called when the widget is removed from the Dashboard
 */
function widgetRemoved()
{
	saveSetting('countryId', null);
	saveSetting('groupId', null);
}

if (window.widget)
{
	widget.onremove = widgetRemoved;
}

