//This document contains class definitions for objects useful in newTunes
AlbumHandler.sortByDate;
AlbumHandler.sortOrder;
AlbumHandler.count = 0;
Artist.count = 0;
Artist.items = [ ];
var nextArtist; //the next artist to be queried on

function Artist(artistData, even){
	status.setArtistCount(++Artist.count);
	this.artistData = artistData;
	this.included = artistData.included;
	//dom manipulation
	this.domEl = document.createElement("div");
	this.domEl.setAttribute("class", even ? 'artistEven' : 'artistOdd');
	this.domEl.innerText = this.artistData.name;
	this.imgEl = document.createElement("img");
	if(this.included){
		this.imgEl.src = "images/artist-included.png";
	}else{
		this.imgEl.src = "images/artist-excluded.png";
	}
	this.imgEl.setAttribute("class", "artistexclude");
	//append image to div
	this.domEl.appendChild(this.imgEl);
	
	//build internal member function
	var self = this;
	this.toggleIncluded = function(){
		self.included = !self.included;
		if(self.included){
			self.imgEl.src = "images/artist-included.png";
		}else{
			self.imgEl.src = "images/artist-excluded.png";
		}
	}
	
	this.domEl.onclick = this.toggleIncluded;
}

Artist.buildList = function(array){
	deleteChildren(artistList);
	delete Artist.items;
	Artist.items = [ ];
	Artist.count = 0;
	for(var i = 0; i < array.length; i++){
		var cur = new Artist(array[i], i % 2 == 0);
		Artist.items.push(cur);
		artistList.appendChild(cur.domEl);
	}
}

/**
 * 
 */
Artist.prototype.isIncluded = function(){
	return this.included;
}

/**
 * 
 */
Artist.prototype.getName = function(){
	return this.artistData.name;
}
/**
 * 
 */
Artist.prototype.toJSON = function(){
	return '{ "included":"' + this.included + '", "name":"' + this.artistData.name + '" }';	
}

Artist.query = function(){
	setTimeout("Artist.queryRecurse(0);", 0);
}

Artist.queryRecurse = function(index){
	//pass the unincluded
	while(Artist.items[index] != null && !Artist.items[index].isIncluded()){ 
		index++;
	}
	if(Artist.items[index] == null){
		searcher.allKeywordsSent();
		return;
	}

	searcher.pushKeyword(Artist.items[index].artistData.name);
	setTimeout("Artist.queryRecurse(" + (index + 1) +");", 0);
}
/**
 * 
 */
Artist.toJSON = function(){
	var artistString = '{"artists" : [ ';
	for(var i = 0; i < Artist.items.length; i++){
		artistString += Artist.items[i].toJSON() + ', ';
	}
	artistString += ' ] }';
	artistString = artistString.replace(/,$/, "");
	return artistString;
}

/**
 * helper functions to assist in group selecting and deselecting
 */
Artist.selectAll = function(){
	for(var i = 0; i < Artist.items.length; i++){
		if(!Artist.items[i].isIncluded()){
			Artist.items[i].toggleIncluded();
		}		
	}
}

Artist.deselectAll = function(){
	for(var i = 0; i < Artist.items.length; i++){
		if(Artist.items[i].isIncluded()){
			Artist.items[i].toggleIncluded();
		}		
	}
}

/**
 * Object to handle incoming albums
 * This object builds a binary tree
 * of albums as they come in and
 * can resort the tree if needed.
 * It also keeps track of albums
 * removed permanently via a kill list object
 * and can serializes itself via the JSON functions
 */
function AlbumHandler(){
	this.items = [ ];
	this.favorites = new FavoriteHandler();
	this.killList = new KillList();
	
	var self = this;
	this.sortAlpha = function(){
		if(AlbumHandler.sortByDate){
			AlbumHandler.sortByDate = false;
			self.items.sort(Album.albumSort);
		}else{
			self.items.reverse();
			AlbumHandler.sortOrder *= -1;
		}
		//rearrange the albums
		widget.setPreferenceForKey("alpha","sort");
		widget.setPreferenceForKey("" + AlbumHandler.sortOrder, "sortOrder");
		self.rehashArray();
	}

	this.sortDate = function(){
		if(!AlbumHandler.sortByDate){
			AlbumHandler.sortByDate = true;
			self.items.sort(Album.albumSort);
		}else{
			self.items.reverse();
			AlbumHandler.sortOrder *= -1;
		}
		//rearrange the albums and write preferences
		widget.setPreferenceForKey("date","sort");
		widget.setPreferenceForKey("" + AlbumHandler.sortOrder, "sortOrder");
		self.rehashArray();
	}
	
	this.clearAll = function(){
		delete self.items;
		self.items = [ ];
		deleteChildren(albumList);
		status.setAlbumCount(0);
		AlbumHandler.count = 0;
		scrollArea.refresh();
	}
}

/**
 * determines if the album should be displayed or not
 */
AlbumHandler.prototype.passFilter = function(cd){
	if(!isNew(cd.date)){
		return false;
	}
	if(this.killList.contains(cd)){
		return false;
	}
	if(this.favorites.contains(cd)){		
		return false;
	}									 
	return true;
}

/**
 * This method iterates through the JSObject created by the
 * by the XSLT transformation and passed in through the
 * amazon call back function.
 */
AlbumHandler.prototype.addAlbums = function(albums){
	//there will be 2 batches, one for each artist.
	for(var i = 0; i < albums.ItemSet.Batch.length - 1; i++){
		var batch = albums.ItemSet.Batch;
		//for each album in each batch
		for(var j = 0; j < batch[i].album.length - 1; j++){
			var cdInfo = albums.ItemSet.Batch[i].album[j];
			//fix potential blank artist
			if(cdInfo.artist == ""){
				cdInfo.artist = batch[i].artist;
			}
			if(cdInfo.image == ""){
				cdInfo.image = "images/no-art.png";
			}
			//does it pass the test to get in?
			if(this.passFilter(cdInfo)){
				this.addAlbum( new Album(cdInfo, firstRun) );
			}
		}
	}
	scrollArea.refresh();
}


/**
 * Conduct binary search on the array to find where it should go
 * Insert it there if it is not already...if it is, mark the current one
 * as old.
 */
AlbumHandler.prototype.addAlbum = function(newAlbum){
	//newAlbum.imgEl.src = "images/no-art.png";
	//newAlbum.cdInfo.image = "images/no-art.png";
	var hi = this.items.length;
	if(hi == 0){
		albumList.appendChild(newAlbum.domEl);
		this.items.push(newAlbum);
		status.setAlbumCount(++AlbumHandler.count);
		return;
	}
	var lo = 0;
	var pos;
	while(hi > lo){
		pos = Math.floor((hi - lo)/2) + lo;
		var order = Album.albumSort(this.items[pos], newAlbum);
		if(order < 0){
			lo = ++pos;
		}else if(order > 0){
			hi = pos;
		}else{
			//we found a match - update album art if necessary.
			if(this.items[pos].imgEl.src = "images/no-art.png"){
				this.items[pos].imgEl.src = newAlbum.cdInfo.image;
				this.items[pos].cdInfo.image = newAlbum.cdInfo.image;
			}
			this.items[pos].domEl.style.border = "1px solid #0014D2";
			return;
		}
	}

	if(pos == 0){
		albumList.insertBefore(newAlbum.domEl, this.items[0].domEl);
	}else{
		albumList.insertBefore(newAlbum.domEl, this.items[pos-1].domEl.nextSibling);
	}
	status.setAlbumCount(++AlbumHandler.count);
	this.items.splice(pos, 0, newAlbum);
}

AlbumHandler.prototype.removeAlbum = function(oldAlbum){
	for(i = 0; i < this.items.length; i++){
		if(this.items[i] == oldAlbum){
			this.items.splice(i,1);
			status.setAlbumCount(--AlbumHandler.count);
			return;
		}
	}
}

/**
 * Returns a serialized string of all albums and excluded albums in JSON
 */
AlbumHandler.prototype.getAlbumsInJSON = function(){
	var source = '{ "cds" : [ ';
	for(var i = 0; i < this.items.length; i++){
		source += this.items[i].toJSON() + ",";
	}
	source = source.replace(/,$/, "");
	//append the kill list
	source += "], " + this.killList.toJSON();
	//append favorite list
	source += ", " + this.favorites.toJSON();
	//append the closing braces
	source += " }";
	return source;
}

AlbumHandler.prototype.cleanOld = function(){
	for(var i = this.items.length - 1; i >= 0; i--){
		if(!isNew(this.items[i].cdInfo.date)){
			albumList.removeChild(this.items[i].domEl);
			this.items.splice(i, 1);
			status.setAlbumCount(--AlbumHandler.count);
		}
	}
	scrollArea.refresh();
}

AlbumHandler.prototype.rehashArray = function(){
	for(var i = 0; i < this.items.length - 1; i++){
		albumList.insertBefore(this.items[i+1].domEl);
	}
}


/**
 * Object to create an album element.
 */
function Album(cdInfo, old){
	this.cdInfo = cdInfo;
	//create the element for the album
	this.domEl = document.createElement("div");
	this.domEl.setAttribute("class", "album");
	if(old){
		//give it a blue border so it doesn't stand out
		this.domEl.style.border = "1px solid #0014D2";	
	}
	//create image element
	this.imgEl = document.createElement("img");
	this.imgEl.setAttribute("class", "album-art");
	this.imgEl.src = cdInfo.image;
	this.imgEl.height = 60;
	this.imgEl.width = 60;
	this.domEl.appendChild(this.imgEl);

	//create text elements
	this.albumDesc = document.createElement("p");
	this.albumDesc.setAttribute("class", "album-text-album");
	this.albumDesc.innerText = cdInfo.title;
	this.domEl.appendChild(this.albumDesc);	
	
	this.artistDesc = document.createElement("p");
	this.artistDesc.setAttribute("class", "album-text-artist");
	this.artistDesc.innerText = cdInfo.artist;
	this.domEl.appendChild(this.artistDesc);	
	
	this.dateDesc = document.createElement("p");
	this.dateDesc.setAttribute("class", "album-text-date");
	this.dateDesc.innerText = textDate(cdInfo.date);
	this.domEl.appendChild(this.dateDesc);	
	
	//create the div to contain the buttons
	this.buttonEl = document.createElement("div");
	this.buttonEl.setAttribute("class", "album-buttons");
	this.domEl.appendChild(this.buttonEl);
	
	//create tool tip box
	this.textEl = document.createElement("span");
	this.textEl.setAttribute("class", "album-button-text");

	//internal functions
	var self = this;
	this.killAlbum = function(){
		albumList.removeChild(self.domEl);
		scrollArea.refresh();
		albumHandler.removeAlbum(self);
		albumHandler.killList.add(self.cdInfo);
	}
	
	this.clearAlbum = function(){
		albumList.removeChild(self.domEl);
		albumHandler.removeAlbum(self);
		scrollArea.refresh();

	}
	
	this.buyAlbum = function(){
		if(widget){
			clicked(self.cdInfo.url);
		}
	}
	
	this.makeFavorite = function(){
		self.textEl.innerText = "";
		albumHandler.favorites.addAlbum(self);
		albumHandler.removeAlbum(self);	
		scrollArea.refresh();
	}
	
	//add the function for the album art if people are so inclined to click it
	this.imgEl.onclick = this.buyAlbum;
	
	//create button functionality
	this.killEl = document.createElement("img");
	this.clearEl = document.createElement("img");
	this.buyEl = document.createElement("img");
	this.favoriteEl = document.createElement("img");
	
	this.killEl.setAttribute("class", "album-button");
	this.clearEl.setAttribute("class", "album-button");
	this.buyEl.setAttribute("class", "album-button");
	this.favoriteEl.setAttribute("class", "album-button");
	
	this.buttonEl.appendChild(this.killEl);
	this.buttonEl.appendChild(this.clearEl);
	this.buttonEl.appendChild(this.buyEl);
	this.buttonEl.appendChild(this.favoriteEl);
	this.buttonEl.appendChild(this.textEl);

	this.killButton = new SimpleButton(this.killEl, 	this.killAlbum, 		"images/alb-kill.png", 		"images/alb-kill-hl.png", 	this.textEl, "never show again");
	this.clearButton = new SimpleButton(this.clearEl, 	this.clearAlbum, 	"images/alb-clear.png", 		"images/alb-clear-hl.png", 	this.textEl, "remove");
	this.buyButton = new SimpleButton(this.buyEl, 		this.buyAlbum, 		"images/alb-buy.png", 		"images/alb-buy-hl.png", 		this.textEl, "buy on amazon.com");
	this.favoriteButton = new SimpleButton(this.favoriteEl, this.makeFavorite, 	"images/alb-favorite.png", 	"images/alb-favorite-hl.png", this.textEl, "add to wish list");
}

/**
 * Serialize the album into javscript object notation
 * to be saved to preferences
 */
Album.prototype.toJSON = function(){
	var cd = this.cdInfo;
	return '{ "artist":"'+ cd.artist + 
			'","title":"' + cd.title +
			'","date":"' + cd.date + 
			'","url":"' 	+ cd.url + 
			'","image":"' + cd.image + '" }';
}

/**
 * Esssentially a sort function for the binary tree to look at as
 * objects are brought into the tree.  It is based on 
 * the sort criteria specified by the user.
 */
Album.albumSort = function(a,b){
	if(AlbumHandler.sortByDate){
		return mySort(a.cdInfo.date + a.cdInfo.artist + a.cdInfo.title, b.cdInfo.date + b.cdInfo.artist + b.cdInfo.title) * AlbumHandler.sortOrder;
	}else{
		return mySort(a.cdInfo.artist + a.cdInfo.title + a.cdInfo.date, b.cdInfo.artist + b.cdInfo.title + b.cdInfo.date) * AlbumHandler.sortOrder;
	}
}

/**
 * Handler to keep track of the items no one wants to see again
 * The albums are stored in a linked list internal to the this type
 * and can be removed from the preferences window.
 */
function KillList() {
	this.items = [ ];
	this.listBox = document.getElementById("excludedList");
}

/**
 * Inserts into the array using binary search for speed
 * and to preserve order
 */
KillList.prototype.add = function(cdInfo){
	//insert the item at the index found by the binary search
	this.items.splice(this.search(cdInfo).index, 0, cdInfo);
}

KillList.prototype.toJSON = function(){
	var ret = '"kill" : [';
	for(var i = 0; i < this.items.length; i++){
		ret += '{ "artist":"'+ this.items[i].artist + '", "title":"' + this.items[i].title + '", "date":"' + this.items[i].date +  '" },';
	}
	//delete any trailing comma...add the last bit
	ret = ret.replace(/,$/, "");
	ret += " ]";
	return ret;
}
 
/**
 * Returns true if the kill list contains the passed in
 * album, meaning it has previously been rejected by the user.
 */
KillList.prototype.contains = function(cdInfo){
	return this.search(cdInfo).contains;
}

KillList.prototype.populateListbox = function(){
	deleteChildren(this.listBox);
	for(var i = 0; i < this.items.length; i++){
		var opt = new Option();
		opt.text = this.items[i].artist + ", " + this.items[i].title + " (" + this.items[i].date +")" ;
		this.listBox.add(opt);
	}
}

/**
 * Conduct binary search on the kill list for the
 * given album info
 * @param {Object} cdInfo - album info object
 */
KillList.prototype.search = function(cdInfo){
	var result = { index: 0, contains: false}
	var hi = this.items.length;
	var lo = 0;
	while(hi > lo){
		result.index = Math.floor((hi - lo)/2) + lo;
		var order = mySort(this.items[result.index].artist + this.items[result.index].title + this.items[result.index].date, cdInfo.artist + cdInfo.title + cdInfo.date);
		if(order < 0){
			lo = ++result.index;
		}else if(order > 0){
			hi = result.index;
		}else{
			//we found a match
			result.contains = true;
			break;
		}
	}
	return result;
}

/**
 * Remove the item selected in the excluded box
 */
KillList.prototype.deleteSelected = function() { 
	this.items.splice(this.listBox.selectedIndex, 1);
	this.listBox.remove(this.listBox.selectedIndex);
}

/**
 * Clear all excluded albums
 */
KillList.prototype.clear = function() { 
	deleteChildren(this.listBox);
	this.items = [ ];
}


function FavoriteHandler(){
	this.items = [ ];
	var self = this;
	
	this.addAlbum = function(album){
		//conduct transformations here
		//change the action to remove the item from favorites
		album.clearButton.setAction(function() { self.remove(album) } );
		album.buttonEl.removeChild(album.favoriteEl);
		album.buttonEl.removeChild(album.killEl);
		self.items.push(album);
		wishList.appendChild(album.domEl);
	}
	
	/**
	 * Serialize the favorites
	 */
	this.toJSON = function(){
		var ret = '"favorite" : [';
		for(var i = 0; i < self.items.length; i++){
			ret += self.items[i].toJSON() + ",";
		}
		//delete any trailing comma...add the last bit
		ret = ret.replace(/,$/, "");
		ret += " ]";		
		return ret;
	}
	
	/**
	 * returns true if the favorites list contains the
	 * given item, false otherwise.
	 * @param {Object} albumInfo
	 */
	this.contains = function(albumInfo){
		for(var i = 0; i < this.items.length; i++){
			if(albumInfo.artist == self.items[i].cdInfo.artist
			   && albumInfo.title == self.items[i].cdInfo.title
			   && albumInfo.date == self.items[i].cdInfo.date){
				//update image if necessary
				if(self.items[i].imgEl.src = "images/no-art.png"){
					self.items[i].imgEl.src = albumInfo.image;
					self.items[i].cdInfo.image = albumInfo.image;
				}
				return true;
			}
		}
		return false;
	}
	
	/**
	 * Removes an item from the favorites list
	 * @param {Object} album
	 */
	this.remove = function(album){
		wishList.removeChild(album.domEl);
		for(var i = 0; i < self.items.length; i++){
			if(self.items[i] == album){
				self.items.splice(i, 1);
				albumHandler.addAlbum(new Album(album.cdInfo, true));
				break;
			}
		}
	}
}
